let diccionario = {};
const mensajesContainer = document.getElementById('mensajes');
const entradaMensaje = document.getElementById('texto-mensaje');
let historialRespuestas = [];

// Ruta del archivo de índice JSON
const rutaIndiceJSON = './chatbot-index.json';

// Función para cargar las intenciones dinámicamente desde el índice
async function cargarIntencionesDinamicamente() {
    try {
        console.log("Cargando archivo de índice...");
        const respuestaIndice = await fetch(rutaIndiceJSON);

        if (!respuestaIndice.ok) {
            throw new Error(`Error al cargar el archivo de índice: ${respuestaIndice.statusText}`);
        }

        const indexData = await respuestaIndice.json();
        console.log("Archivo de índice cargado:", indexData);

        const rutas = Object.values(indexData); // Extraer rutas desde el índice

        const promesas = rutas.map(async (ruta) => {
            console.log(`Cargando archivo: ${ruta}`);
            try {
                const respuestaArchivo = await fetch(ruta);
                if (!respuestaArchivo.ok) {
                    throw new Error(`Error al cargar el archivo ${ruta}: ${respuestaArchivo.statusText}`);
                }
                const datos = await respuestaArchivo.json();

                // Validar que los datos tengan la estructura esperada
                if (datos.palabras && datos.respuestas) {
                    const nombreContexto = ruta.split('/').pop().replace('.json', '');
                    diccionario[nombreContexto] = datos;
                    historialRespuestas[nombreContexto] = [];
                    console.log(`Cargado: ${nombreContexto}`);
                } else {
                    console.warn(`El archivo ${ruta} no tiene la estructura esperada.`);
                }
            } catch (error) {
                console.error(`Error procesando el archivo ${ruta}:`, error);
            }
        });

        await Promise.all(promesas);
        console.log("Intenciones cargadas:", Object.keys(diccionario));

        // Mostrar mensaje de bienvenida
        const mensajeBienvenida = obtenerRespuestaUnica('conocenos') || '### ¡Hola soy tu asistente virtual! \n\n**¿En qué puedo ayudarte?**';
        mostrarMensaje('bot', mensajeBienvenida);
    } catch (error) {
        console.error("Error cargando las intenciones dinámicamente:", error);
        mostrarMensaje('bot', '### **¡Lo siento, hubo un error al cargar la información!**\n\nPor favor, inténtalo de nuevo más tarde...');
    }
}

// Limpiar texto de entrada
function limpiarTexto(texto) {
    return texto
        .toLowerCase()
        .normalize('NFD')
        .replace(/[\u0300-\u036f]/g, '')
        .replace(/[¿?.;,_+*/\|!¡]/g, '')
        .trim();
}

// Encontrar palabras clave en el mensaje
function encontrarPalabrasClave(texto) {
    texto = limpiarTexto(texto);
    const palabrasEncontradas = Object.keys(diccionario).filter((categoria) => {
        const palabrasClave = diccionario[categoria]?.palabras || [];
        return palabrasClave.some((palabra) => texto.includes(palabra));
    });

    if (palabrasEncontradas.includes('saludos')) {
        return ['saludos'];
    }

    return [...new Set(palabrasEncontradas)]; // Eliminar duplicados
}

// Obtener respuesta única basada en la categoría
function obtenerRespuestaUnica(categoria) {
    const respuestasCategoria = diccionario[categoria]?.respuestas || [];
    if (respuestasCategoria.length === 0) {
        return '**No se encontraron respuestas para esta categoría.**';
    }

    const respuestasDisponibles = respuestasCategoria.filter((r) => !historialRespuestas.includes(r));

    let respuesta;
    if (respuestasDisponibles.length > 0) {
        respuesta = respuestasDisponibles[Math.floor(Math.random() * respuestasDisponibles.length)];
    } else {
        respuesta = respuestasCategoria[Math.floor(Math.random() * respuestasCategoria.length)];
    }

    historialRespuestas.push(respuesta);
    return respuesta;
}

// Función para enviar un mensaje desde el input
function enviarMensaje() {
    const mensaje = entradaMensaje.value;
    if (!mensaje) return;

    manejarMensajeUsuario(mensaje);
    entradaMensaje.value = '';
}

// Función para manejar un mensaje (input o botón)
function manejarMensajeUsuario(mensaje) {
    console.log("Mensaje enviado al chatbot:", mensaje);

    mostrarMensaje('usuario', mensaje);

    const palabrasClave = encontrarPalabrasClave(mensaje);
    const respuestasGeneradas = new Set();

    if (palabrasClave.length > 0) {
        palabrasClave.forEach((categoria) => {
            const respuesta = obtenerRespuestaUnica(categoria);
            if (!respuestasGeneradas.has(respuesta)) {
                respuestasGeneradas.add(respuesta);
                mostrarMensaje('bot', respuesta);
            }
        });
    } else {
        mostrarMensaje('bot', '**Lo siento, no comprendo completamente tu mensaje.**\n\n¿Podrías probar con otros términos?');
    }
}

// Mostrar el mensaje en la interfaz
function mostrarMensaje(tipo, texto) {
    const mensajeDiv = document.createElement('div');
    mensajeDiv.classList.add(tipo);

    // Usamos DOMPurify para limpiar el texto
    const sanitizedHTML = DOMPurify.sanitize(marked.parse(texto), {
        ALLOW_TAGS: ['iframe', 'button', 'div', 'img', 'a', 'p', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'br', 'strong', 'em'],
        ALLOW_ATTR: ['src', 'href', 'class', 'onclick', 'frameborder', 'allowfullscreen', 'alt', 'width', 'height'],
    });

    mensajeDiv.innerHTML = sanitizedHTML;

    // Agregar el nuevo mensaje al contenedor de mensajes
    mensajesContainer.appendChild(mensajeDiv);
    mensajesContainer.scrollTop = mensajesContainer.scrollHeight;
}

// Función global para enviar mensajes desde botones
function chatbotEnviarMensaje(mensaje) {
    if (!mensaje) {
        console.error("Mensaje vacío o no válido recibido.");
        return;
    }
    console.log("Botón clickeado. Mensaje:", mensaje);
    manejarMensajeUsuario(mensaje); 
}



// Agregar el evento al botón "Enviar"
document.querySelector('button').addEventListener('click', enviarMensaje);
mensajesContainer.addEventListener('click', function(event) {
    if (event.target.tagName === 'BUTTON') {
        const mensaje = event.target.getAttribute('onclick').match(/chatbotEnviarMensaje\("(.+?)"\)/)?.[1];
        if (mensaje) {
            chatbotEnviarMensaje(mensaje);
        }
    }
});


// Agregar el evento para enviar mensaje al presionar "Enter"
entradaMensaje.addEventListener('keydown', function (event) {
    if (event.key === 'Enter') {
        event.preventDefault(); // Evitar comportamiento predeterminado
        enviarMensaje(); // Llamar a la función que envía el mensaje
    }
});

// Inicializar el chatbot
window.onload = async () => {
    entradaMensaje.focus();
    await cargarIntencionesDinamicamente();
};
